module HousingMisc
  class SidekiqQueueCheckController < ApplicationController 
    include HousingMisc::ApiHelper
    before_action :internal_host_check
    DEFAULT_QUEUE_SIZE = 100
    DEFAULT_TOTAL_SIZE = 1000

    def check_permission
      return false
    end

    def check
      return unless check_sidekiq_presence
      require "sidekiq/api"

      jobs_info = get_above_threshold_queue_list 
      alert_slack(HousingMisc.sidekiq_message_url, get_message(jobs_info)) unless jobs_info.empty?
      jobs_info = {"message" => "no sidekiq queue above threshold"} if jobs_info.empty?
      render json: jobs_info
    end

    def get_above_threshold_queue_list
      jobs_info = {}
      stats = Sidekiq::Stats.new
      total_threshold = HousingMisc.sidekiq_custom_threshold["total"] || DEFAULT_TOTAL_SIZE
      if stats.enqueued > total_threshold
        jobs_info["total"] = {"jobs" => stats.enqueued, "threshold" => total_threshold } 
      end
      stats.queues.each do |key, value|
        queue_threshold = HousingMisc.sidekiq_custom_threshold[key] || DEFAULT_QUEUE_SIZE
        if value > queue_threshold
          jobs_info[key] = {"jobs" => value, "threshold" => queue_threshold } 
        end
      end
      jobs_info
    end

    def get_message(jobs_info)
      message = "#{Time.now.to_s} :: Queues with jobs exceeding threshold in #{Rails.application.class.parent_name} service => "
      jobs_info.each do |key, value|
        message += " #{key} : #{value["jobs"]} jobs (Threshold : #{value["threshold"]}), "
      end
      message
    end

    def check_sidekiq_presence
      return true if Gem.loaded_specs["sidekiq"].present?
      render json: {"message" => "no sidekiq in this service"}
      return false
    end

    def alert_slack(url, message)
      url = URI(url)
      req = Net::HTTP::Post.new url.path
      req.body = {:text => message}.to_json
      res = Net::HTTP.start(url.host, url.port, :use_ssl => true) do |http|
        http.request req
      end
    end

  end
end